"""
This module provides methods to get/set the (IPv4) configuration of the eth0 interface
via /etc/network/interfaces.

Warning:
    This code assumes that the /etc/network/interfaces file starts at a sane default.
    We don't try to rewrite the whole thing every time, because there may be other interfaces
    configured by hand besides eth0. We just munge /etc/network/interfaces and
    issue ifdown/ifup commands.

"""

import logging
import socket
import subprocess
import sys
import time

if sys.platform.startswith("win"):  # pragma: no cover
    pass
elif sys.platform.startswith("linux") or sys.platform == 'darwin':
    import fcntl
    import struct

from .e10_version import get_e10_version

log = logging.getLogger(__name__)


E10_VERSION = None

try:
    E10_VERSION = get_e10_version()
except:
    E10_VERSION = None

SIOCGIFADDR = 0x8915
SIOCGIFHWADDR = 0x8927
SIOCGIFNETMASK = 0x891b

INTERFACES = '/etc/network/interfaces'
IF_NAME = 'eth0'


def get_eth0_config():
    """
    Returns a dictionary with the current eth0 configuration.

    Returns:
        Dictionary with keys:
            'dhcp' = True or False
            'ip_addr' = current IPv4 IP Address
            'netmask' = current IPv4 Netmask
            'gateway' = current IPv4 default gateway
            'mac_addr' = current eth0 mac address
    """

    config = {}
    config['dhcp'] = _check_for_dhcp(IF_NAME)
    config['ip_addr'] = _get_ip_addr(IF_NAME)
    config['netmask'] = _get_netmask(IF_NAME)
    config['gateway'] = _get_gateway(IF_NAME)
    config['mac_addr'] = _get_mac_addr(IF_NAME)
    config['hostname'] = socket.gethostname()

    return config


def set_eth0_config(dhcp, ip_addr='', netmask='', gateway=''):  # pragma: no cover
    """
    Alters an E10's network configuration by sed-munging /etc/network/interfaces.
    Checks for e10-ness first, and will raise a NotAnE10Exception.

    Note:
        If you're setting dhcp to True, you don't have to pass ip/netmask/gateway.

    Args:
        dhcp: Boolean (True = DHCP, False = Static)
        ip_addr: IPv4 Address string (xxx.xxx.xxx.xxx)
        netmask: IPv4 Netmask string (xxx.xxx.xxx.xxx)
        gateway: IPv4 Gateway string (xxx.xxx.xxx.xxx)

    Raises:
        NotAnE10Exception

    """

    # we don't care which version we are, just that we're an e10.
    # (This will raise a NotAnE10Exception on non-E10s.)
    e10version = get_e10_version()
    log.debug("Running on a %s E10", e10version)

    if dhcp:
        _go_dhcp()
    elif ip_addr and netmask and gateway and len(ip_addr) > 0 and len(netmask) > 0 and len(gateway) > 0:
        _go_static(ip_addr, netmask, gateway)

    # Give the system time for things to settle
    # Yes, time.sleep is Bad, Evil, and Wrong.
    # Don't call set_eth0_config in a tight loop, m'kay?
    time.sleep(1)
    subprocess.call(['ifdown', 'eth0'])

    # For whatever reason, the ifdown call above doesn't do
    # the right thing and kill off the udhcpc process for eth0.
    # Therefore, we'll try to kill it with fire^W^Wby pidfile.
    try:
        with open('/var/run/udhcpc.eth0.pid', 'r') as pidfile:
            pid = pidfile.readline().strip()
            log.debug("about to kill %s", pid)
            subprocess.call(['kill', pid])
    except IOError:
        log.exception("Couldn't kill udhcpc, could just be a bad pidfile (or it's not running)")

    time.sleep(3)
    subprocess.call(['ifup', 'eth0'])


def _go_static(ip_addr, netmask, gateway, interfaces=INTERFACES):  # pragma: no cover
    """Change the eth0 config in /etc/network/interfaces to a static IP.
    Uses the ip_addr, netmask, and gateway provided.
    """

    lines = []
    with open(interfaces, 'rt') as f:
        lines = f.readlines()

    log.debug("Lines: %s", lines)

    with open(interfaces, 'wt+') as f:

        in_static_block = False

        for line in lines:
            if 'iface ' + IF_NAME + ' inet dhcp' in line:
                if line[0] != '#':
                    f.write('#' + line)
                else:
                    f.write(line)
            elif 'iface ' + IF_NAME + ' inet static' in line:
                in_static_block = True
                f.write('iface ' + IF_NAME + ' inet static\n')
            elif in_static_block and 'address' in line:
                f.write('address ' + ip_addr + '\n')
            elif in_static_block and 'netmask' in line:
                f.write('netmask ' + netmask + '\n')
            elif in_static_block and 'gateway' in line:
                f.write('gateway ' + gateway + '\n')
                in_static_block = False
            else:
                f.write(line)

    subprocess.call(['sync'])


def _go_dhcp(interfaces=INTERFACES):  # pragma: no cover
    """Change the eth0 config in /etc/network/interfaces to DHCP."""
    # uncomment the dhcp line in /etc/network/interfaces

    lines = []
    with open(interfaces, 'rt') as f:
        lines = f.readlines()

    log.debug("Lines: %s", lines)

    with open(interfaces, 'wt+') as f:

        in_static_block = False

        for line in lines:
            if 'iface ' + IF_NAME + ' inet dhcp' in line:
                f.write('iface ' + IF_NAME + ' inet dhcp\n')
            elif 'iface ' + IF_NAME + ' inet static' in line:
                in_static_block = True
                if line[0] != '#':
                    f.write('#' + line)
                else:
                    f.write(line)
            elif in_static_block and 'address' in line:
                if line[0] != '#':
                    f.write('#' + line)
                else:
                    f.write(line)
            elif in_static_block and 'netmask' in line:
                if line[0] != '#':
                    f.write('#' + line)
                else:
                    f.write(line)
            elif in_static_block and 'gateway' in line:
                if line[0] != '#':
                    f.write('#' + line)
                else:
                    f.write(line)
                in_static_block = False
            else:
                f.write(line)

    subprocess.call(['sync'])


def _get_ip_addr(if_name):  # pragma: no cover
    """Retrieves the current IP address."""

    if E10_VERSION is None:
        return ""

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        SIOCGIFADDR,
        struct.pack('256s', if_name[:15]))[20:24])

def _get_mac_addr(if_name):  # pragma: no cover
    """Retrieve the mac address for the interface"""

    if E10_VERSION is None:
        return ""

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), SIOCGIFHWADDR, struct.pack('256s', if_name[:15]))
    return "".join(['%02x:' % ord(char) for char in info[18:24]])[:-1]


def _get_netmask(if_name):  # pragma: no cover
    """Retrieves the current netmask."""

    if E10_VERSION is None:
        return ""

    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    netmask = fcntl.ioctl(s, SIOCGIFNETMASK, struct.pack('256s', if_name))[20:24]
    return socket.inet_ntoa(netmask)


def _get_gateway(if_name):  # pragma: no cover
    """Retrieves the current gateway."""

    if E10_VERSION is None:
        return ""

    first = ['ip', 'route', 'show', 'dev', if_name]
    second = ['awk', ' /^default/ {print $3}']
    p1 = subprocess.Popen(first, stdout=subprocess.PIPE)
    p2 = subprocess.Popen(second, stdin=p1.stdout, stdout=subprocess.PIPE)
    p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
    output = p2.communicate()[0]
    return output.strip()


def _check_for_dhcp(if_name):  # pragma: no cover
    """Looks for udhcpc running for eth0. If it's running, we assume we're using DHCP."""

    if E10_VERSION is None:
        return False

    first = ['ps']
    second = ['grep', 'dhcp.*' + if_name, '-c']
    p1 = subprocess.Popen(first, stdout=subprocess.PIPE)
    p2 = subprocess.Popen(second, stdin=p1.stdout, stdout=subprocess.PIPE)
    p1.stdout.close()  # Allow p1 to receive a SIGPIPE if p2 exits.
    output = p2.communicate()[0]
    # If there's more than 1 process that matches the grep,
    # then udhcpc is running (and we're grepping for it.)
    # At least, that's what we're assuming.
    return int(output.strip()) > 1
